@@ -1,21 +1,48 @@
-import asyncio
-import email.utils
+from __future__ import annotations
 
-from collections import defaultdict, namedtuple
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    DefaultDict,
+    Dict,
+    List,
+    NamedTuple,
+    Optional,
+    Tuple,
+    Union,
+)
+
+from sanic_routing.route import Route  # type: ignore
+
+
+if TYPE_CHECKING:
+    from sanic.server import ConnInfo
+    from sanic.app import Sanic
+    from sanic.http import Http
+
+import email.utils
+import uuid
+
+from collections import defaultdict
 from http.cookies import SimpleCookie
 from types import SimpleNamespace
 from urllib.parse import parse_qs, parse_qsl, unquote, urlunparse
 
 from httptools import parse_url  # type: ignore
 
+from sanic.compat import CancelledErrors, Header
+from sanic.constants import DEFAULT_HTTP_CONTENT_TYPE
 from sanic.exceptions import InvalidUsage
 from sanic.headers import (
+    Options,
     parse_content_header,
     parse_forwarded,
     parse_host,
     parse_xforwarded,
 )
 from sanic.log import error_logger, logger
+from sanic.models.protocol_types import TransportProtocol
+from sanic.response import BaseHTTPResponse, HTTPResponse
 
 
 try:
@@ -23,73 +50,48 @@ try:
 except ImportError:
     from json import loads as json_loads  # type: ignore
 
-DEFAULT_HTTP_CONTENT_TYPE = "application/octet-stream"
-EXPECT_HEADER = "EXPECT"
-
-# HTTP/1.1: https://www.w3.org/Protocols/rfc2616/rfc2616-sec7.html#sec7.2.1
-# > If the media type remains unknown, the recipient SHOULD treat it
-# > as type "application/octet-stream"
-
 
 class RequestParameters(dict):
-    """Hosts a dict with lists as values where get returns the first
+    """
+    Hosts a dict with lists as values where get returns the first
     value of the list and getlist returns the whole shebang
     """
 
-    def get(self, name, default=None):
+    def get(self, name: str, default: Optional[Any] = None) -> Optional[Any]:
         """Return the first value, either the default or actual"""
         return super().get(name, [default])[0]
 
-    def getlist(self, name, default=None):
-        """Return the entire list"""
+    def getlist(
+        self, name: str, default: Optional[Any] = None
+    ) -> Optional[Any]:
+        """
+        Return the entire list
+        """
         return super().get(name, default)
 
 
-class StreamBuffer:
-    def __init__(self, buffer_size=100):
-        self._queue = asyncio.Queue(buffer_size)
-
-    async def read(self):
-        """Stop reading when gets None"""
-        payload = await self._queue.get()
-        self._queue.task_done()
-        return payload
-
-    async def __aiter__(self):
-        """Support `async for data in request.stream`"""
-        while True:
-            data = await self.read()
-            if not data:
-                break
-            yield data
-
-    async def put(self, payload):
-        await self._queue.put(payload)
-
-    def is_full(self):
-        return self._queue.full()
-
-    @property
-    def buffer_size(self):
-        return self._queue.maxsize
-
-
 class Request:
-    """Properties of an HTTP request such as URL, headers, etc."""
+    """
+    Properties of an HTTP request such as URL, headers, etc.
+    """
 
     __slots__ = (
         "__weakref__",
         "_cookies",
+        "_id",
         "_ip",
         "_parsed_url",
         "_port",
+        "_protocol",
         "_remote_addr",
         "_socket",
+        "_match_info",
+        "_name",
         "app",
         "body",
         "conn_info",
         "ctx",
-        "endpoint",
+        "head",
         "headers",
         "method",
         "parsed_args",
@@ -99,72 +101,183 @@ class Request:
         "parsed_json",
         "parsed_forwarded",
         "raw_url",
+        "request_middleware_started",
+        "route",
         "stream",
         "transport",
-        "uri_template",
         "version",
     )
 
-    def __init__(self, url_bytes, headers, version, method, transport, app):
+    def __init__(
+        self,
+        url_bytes: bytes,
+        headers: Header,
+        version: str,
+        method: str,
+        transport: TransportProtocol,
+        app: Sanic,
+        head: bytes = b"",
+    ):
         self.raw_url = url_bytes
         # TODO: Content-Encoding detection
         self._parsed_url = parse_url(url_bytes)
+        self._id: Optional[Union[uuid.UUID, str, int]] = None
+        self._name: Optional[str] = None
         self.app = app
 
         self.headers = headers
         self.version = version
         self.method = method
         self.transport = transport
+        self.head = head
 
         # Init but do not inhale
-        self.body_init()
-        self.conn_info = None
+        self.body = b""
+        self.conn_info: Optional[ConnInfo] = None
         self.ctx = SimpleNamespace()
-        self.parsed_forwarded = None
+        self.parsed_forwarded: Optional[Options] = None
         self.parsed_json = None
         self.parsed_form = None
         self.parsed_files = None
-        self.parsed_args = defaultdict(RequestParameters)
-        self.parsed_not_grouped_args = defaultdict(list)
-        self.uri_template = None
-        self._cookies = None
-        self.stream = None
-        self.endpoint = None
+        self.parsed_args: DefaultDict[
+            Tuple[bool, bool, str, str], RequestParameters
+        ] = defaultdict(RequestParameters)
+        self.parsed_not_grouped_args: DefaultDict[
+            Tuple[bool, bool, str, str], List[Tuple[str, str]]
+        ] = defaultdict(list)
+        self.request_middleware_started = False
+        self._cookies: Optional[Dict[str, str]] = None
+        self._match_info: Dict[str, Any] = {}
+        self.stream: Optional[Http] = None
+        self.route: Optional[Route] = None
+        self._protocol = None
 
     def __repr__(self):
         class_name = self.__class__.__name__
         return f"<{class_name}: {self.method} {self.path}>"
 
-    def body_init(self):
-        """.. deprecated:: 20.3
-        To be removed in 21.3"""
-        self.body = []
+    @classmethod
+    def generate_id(*_):
+        return uuid.uuid4()
 
-    def body_push(self, data):
-        """.. deprecated:: 20.3
-        To be removed in 21.3"""
-        self.body.append(data)
-
-    def body_finish(self):
-        """.. deprecated:: 20.3
-        To be removed in 21.3"""
-        self.body = b"".join(self.body)
+    async def respond(
+        self,
+        response: Optional[BaseHTTPResponse] = None,
+        *,
+        status: int = 200,
+        headers: Optional[Union[Header, Dict[str, str]]] = None,
+        content_type: Optional[str] = None,
+    ):
+        # This logic of determining which response to use is subject to change
+        if response is None:
+            response = (self.stream and self.stream.response) or HTTPResponse(
+                status=status,
+                headers=headers,
+                content_type=content_type,
+            )
+        # Connect the response
+        if isinstance(response, BaseHTTPResponse) and self.stream:
+            response = self.stream.respond(response)
+        # Run response middleware
+        try:
+            response = await self.app._run_response_middleware(
+                self, response, request_name=self.name
+            )
+        except CancelledErrors:
+            raise
+        except Exception:
+            error_logger.exception(
+                "Exception occurred in one of response middleware handlers"
+            )
+        return response
 
     async def receive_body(self):
         """Receive request.body, if not already received.
 
-        Streaming handlers may call this to receive the full body.
+        Streaming handlers may call this to receive the full body. Sanic calls
+        this function before running any handlers of non-streaming routes.
 
-        This is added as a compatibility shim in Sanic 20.3 because future
-        versions of Sanic will make all requests streaming and will use this
-        function instead of the non-async body_init/push/finish functions.
-
-        Please make an issue if your code depends on the old functionality and
-        cannot be upgraded to the new API.
+        Custom request classes can override this for custom handling of both
+        streaming and non-streaming routes.
         """
-        if not self.stream:
-            return
-        self.body = b"".join([data async for data in self.stream])
+        if not self.body:
+            self.body = b"".join([data async for data in self.stream])
+
+    @property
+    def name(self):
+        if self._name:
+            return self._name
+        elif self.route:
+            return self.route.name
+        return None
+
+    @property
+    def endpoint(self):
+        return self.name
+
+    @property
+    def uri_template(self):
+        return f"/{self.route.path}"
+
+    @property
+    def protocol(self):
+        if not self._protocol:
+            self._protocol = self.transport.get_protocol()
+        return self._protocol
+
+    @property
+    def raw_headers(self):
+        _, headers = self.head.split(b"\r\n", 1)
+        return bytes(headers)
+
+    @property
+    def request_line(self):
+        reqline, _ = self.head.split(b"\r\n", 1)
+        return bytes(reqline)
+
+    @property
+    def id(self) -> Optional[Union[uuid.UUID, str, int]]:
+        """
+        A request ID passed from the client, or generated from the backend.
+
+        By default, this will look in a request header defined at:
+        ``self.app.config.REQUEST_ID_HEADER``. It defaults to
+        ``X-Request-ID``. Sanic will try to cast the ID into a ``UUID`` or an
+        ``int``. If there is not a UUID from the client, then Sanic will try
+        to generate an ID by calling ``Request.generate_id()``. The default
+        behavior is to generate a ``UUID``. You can customize this behavior
+        by subclassing ``Request``.
+
+        .. code-block:: python
+
+            from sanic import Request, Sanic
+            from itertools import count
+
+            class IntRequest(Request):
+                counter = count()
+
+                def generate_id(self):
+                    return next(self.counter)
+
+            app = Sanic("MyApp", request_class=IntRequest)
+        """
+        if not self._id:
+            self._id = self.headers.get(
+                self.app.config.REQUEST_ID_HEADER,
+                self.__class__.generate_id(self),  # type: ignore
+            )
+
+            # Try casting to a UUID or an integer
+            if isinstance(self._id, str):
+                try:
+                    self._id = uuid.UUID(self._id)
+                except ValueError:
+                    try:
+                        self._id = int(self._id)  # type: ignore
+                    except ValueError:
+                        ...
+
+        return self._id  # type: ignore
 
     @property
     def json(self):
@@ -265,12 +378,9 @@ class Request:
         :type errors: str
         :return: RequestParameters
         """
-        if (
-            keep_blank_values,
-            strict_parsing,
-            encoding,
-            errors,
-        ) not in self.parsed_args:
+        if not self.parsed_args[
+            (keep_blank_values, strict_parsing, encoding, errors)
+        ]:
             if self.query_string:
                 self.parsed_args[
                     (keep_blank_values, strict_parsing, encoding, errors)
@@ -324,12 +434,9 @@ class Request:
         :type errors: str
         :return: list
         """
-        if (
-            keep_blank_values,
-            strict_parsing,
-            encoding,
-            errors,
-        ) not in self.parsed_not_grouped_args:
+        if not self.parsed_not_grouped_args[
+            (keep_blank_values, strict_parsing, encoding, errors)
+        ]:
             if self.query_string:
                 self.parsed_not_grouped_args[
                     (keep_blank_values, strict_parsing, encoding, errors)
@@ -345,13 +452,22 @@ class Request:
         ]
 
     query_args = property(get_query_args)
+    """
+    Convenience property to access :meth:`Request.get_query_args` with
+    default values.
+    """
 
     @property
-    def cookies(self):
+    def cookies(self) -> Dict[str, str]:
+        """
+        :return: Incoming cookies on the request
+        :rtype: Dict[str, str]
+        """
+
         if self._cookies is None:
             cookie = self.headers.get("Cookie")
             if cookie is not None:
-                cookies = SimpleCookie()
+                cookies: SimpleCookie = SimpleCookie()
                 cookies.load(cookie)
                 self._cookies = {
                     name: cookie.value for name, cookie in cookies.items()
@@ -361,27 +477,35 @@ class Request:
         return self._cookies
 
     @property
-    def content_type(self):
+    def content_type(self) -> str:
+        """
+        :return: Content-Type header form the request
+        :rtype: str
+        """
         return self.headers.get("Content-Type", DEFAULT_HTTP_CONTENT_TYPE)
 
     @property
     def match_info(self):
-        """return matched info after resolving route"""
-        return self.app.router.get(self)[2]
+        """
+        :return: matched info after resolving route
+        """
+        return self._match_info
 
     # Transport properties (obtained from local interface only)
 
     @property
-    def ip(self):
+    def ip(self) -> str:
         """
         :return: peer ip of the socket
+        :rtype: str
         """
         return self.conn_info.client if self.conn_info else ""
 
     @property
-    def port(self):
+    def port(self) -> int:
         """
         :return: peer port of the socket
+        :rtype: int
         """
         return self.conn_info.client_port if self.conn_info else 0
 
@@ -391,13 +515,16 @@ class Request:
 
     @property
     def path(self) -> str:
-        """Path of the local HTTP request."""
+        """
+        :return: path of the local HTTP request
+        :rtype: str
+        """
         return self._parsed_url.path.decode("utf-8")
 
     # Proxy properties (using SERVER_NAME/forwarded/request/transport info)
 
     @property
-    def forwarded(self):
+    def forwarded(self) -> Options:
         """
         Active proxy information obtained from request headers, as specified in
         Sanic configuration.
@@ -408,6 +535,9 @@ class Request:
         - path is url-unencoded
 
         Additional values may be available from new style Forwarded headers.
+
+        :return: forwarded address info
+        :rtype: Dict[str, str]
         """
         if self.parsed_forwarded is None:
             self.parsed_forwarded = (
@@ -423,10 +553,14 @@ class Request:
         Client IP address, if available.
         1. proxied remote address `self.forwarded['for']`
         2. local remote address `self.ip`
+
         :return: IPv4, bracketed IPv6, UNIX socket name or arbitrary string
+        :rtype: str
         """
         if not hasattr(self, "_remote_addr"):
-            self._remote_addr = self.forwarded.get("for", "")  # or self.ip
+            self._remote_addr = str(
+                self.forwarded.get("for", "")
+            )  # or self.ip
         return self._remote_addr
 
     @property
@@ -436,12 +570,14 @@ class Request:
         1. `config.SERVER_NAME` if in full URL format
         2. proxied proto/scheme
         3. local connection protocol
+
         :return: http|https|ws|wss or arbitrary value given by the headers.
+        :rtype: str
         """
         if "//" in self.app.config.get("SERVER_NAME", ""):
             return self.app.config.SERVER_NAME.split("//")[0]
         if "proto" in self.forwarded:
-            return self.forwarded["proto"]
+            return str(self.forwarded["proto"])
 
         if (
             self.app.websocket_enabled
@@ -465,48 +601,67 @@ class Request:
         3. request host header
         hostname and port may be separated by
         `sanic.headers.parse_host(request.host)`.
+
         :return: the first matching host found, or empty string
+        :rtype: str
         """
         server_name = self.app.config.get("SERVER_NAME")
         if server_name:
             return server_name.split("//", 1)[-1].split("/", 1)[0]
-        return self.forwarded.get("host") or self.headers.get("host", "")
+        return str(self.forwarded.get("host") or self.headers.get("host", ""))
 
     @property
     def server_name(self) -> str:
-        """The hostname the client connected to, by `request.host`."""
+        """
+        :return: hostname the client connected to, by ``request.host``
+        :rtype: str
+        """
         return parse_host(self.host)[0] or ""
 
     @property
     def server_port(self) -> int:
         """
-        The port the client connected to, by forwarded `port` or
-        `request.host`.
+        The port the client connected to, by forwarded ``port`` or
+        ``request.host``.
 
-        Default port is returned as 80 and 443 based on `request.scheme`.
+        Default port is returned as 80 and 443 based on ``request.scheme``.
+
+        :return: port number
+        :rtype: int
         """
         port = self.forwarded.get("port") or parse_host(self.host)[1]
-        return port or (80 if self.scheme in ("http", "ws") else 443)
+        return int(port or (80 if self.scheme in ("http", "ws") else 443))
 
     @property
     def server_path(self) -> str:
-        """Full path of current URL. Uses proxied or local path."""
-        return self.forwarded.get("path") or self.path
+        """
+        :return: full path of current URL; uses proxied or local path
+        :rtype: str
+        """
+        return str(self.forwarded.get("path") or self.path)
 
     @property
-    def query_string(self):
+    def query_string(self) -> str:
+        """
+        :return: representation of the requested query
+        :rtype: str
+        """
         if self._parsed_url.query:
             return self._parsed_url.query.decode("utf-8")
         else:
             return ""
 
     @property
-    def url(self):
+    def url(self) -> str:
+        """
+        :return: the URL
+        :rtype: str
+        """
         return urlunparse(
             (self.scheme, self.host, self.path, None, self.query_string, None)
         )
 
-    def url_for(self, view_name, **kwargs):
+    def url_for(self, view_name: str, **kwargs) -> str:
         """
         Same as :func:`sanic.Sanic.url_for`, but automatically determine
         `scheme` and `netloc` base on the request. Since this method is aiming
@@ -539,11 +694,24 @@ class Request:
         )
 
 
-File = namedtuple("File", ["type", "body", "name"])
+class File(NamedTuple):
+    """
+    Model for defining a file. It is a ``namedtuple``, therefore you can
+    iterate over the object, or access the parameters by name.
+
+    :param type: The mimetype, defaults to text/plain
+    :param body: Bytes of the file
+    :param name: The filename
+    """
+
+    type: str
+    body: bytes
+    name: str
 
 
 def parse_multipart_form(body, boundary):
-    """Parse a request body and returns fields and files
+    """
+    Parse a request body and returns fields and files
 
     :param body: bytes request body
     :param boundary: bytes multipart boundary
