@@ -4,10 +4,15 @@ import string
 import sys
 import uuid
 
+from typing import Tuple
+
 import pytest
 
+from sanic_routing.exceptions import RouteExists
+
 from sanic import Sanic
-from sanic.router import RouteExists, Router
+from sanic.constants import HTTP_METHODS
+from sanic.router import Router
 
 
 random.seed("Pack my box with five dozen liquor jugs.")
@@ -17,6 +22,11 @@ if sys.platform in ["win32", "cygwin"]:
     collect_ignore = ["test_worker.py"]
 
 
+@pytest.fixture
+def caplog(caplog):
+    yield caplog
+
+
 async def _handler(request):
     """
     Dummy placeholder method used for route resolver when creating a new
@@ -31,12 +41,12 @@ async def _handler(request):
 
 TYPE_TO_GENERATOR_MAP = {
     "string": lambda: "".join(
-        [random.choice(string.ascii_letters + string.digits) for _ in range(4)]
+        [random.choice(string.ascii_lowercase) for _ in range(4)]
     ),
     "int": lambda: random.choice(range(1000000)),
     "number": lambda: random.random(),
     "alpha": lambda: "".join(
-        [random.choice(string.ascii_letters) for _ in range(4)]
+        [random.choice(string.ascii_lowercase) for _ in range(4)]
     ),
     "uuid": lambda: str(uuid.uuid1()),
 }
@@ -45,7 +55,7 @@ TYPE_TO_GENERATOR_MAP = {
 class RouteStringGenerator:
 
     ROUTE_COUNT_PER_DEPTH = 100
-    HTTP_METHODS = ["GET", "PUT", "POST", "PATCH", "DELETE", "OPTION"]
+    HTTP_METHODS = HTTP_METHODS
     ROUTE_PARAM_TYPES = ["string", "int", "number", "alpha", "uuid"]
 
     def generate_random_direct_route(self, max_route_depth=4):
@@ -98,12 +108,12 @@ class RouteStringGenerator:
 @pytest.fixture(scope="function")
 def sanic_router(app):
     # noinspection PyProtectedMember
-    def _setup(route_details: tuple) -> (Router, tuple):
-        router = Router(app)
+    def _setup(route_details: tuple) -> Tuple[Router, tuple]:
+        router = Router()
         added_router = []
         for method, route in route_details:
             try:
-                router._add(
+                router.add(
                     uri=f"/{route}",
                     methods=frozenset({method}),
                     host="localhost",
@@ -112,6 +122,7 @@ def sanic_router(app):
                 added_router.append((method, route))
             except RouteExists:
                 pass
+        router.finalize()
         return router, added_router
 
     return _setup
@@ -127,6 +138,7 @@ def url_param_generator():
     return TYPE_TO_GENERATOR_MAP
 
 
-@pytest.fixture
+@pytest.fixture(scope="function")
 def app(request):
-    return Sanic(request.node.name)
+    app = Sanic(request.node.name)
+    return app
