@@ -7,7 +7,12 @@ import pytest
 from sanic.app import Sanic
 from sanic.blueprints import Blueprint
 from sanic.constants import HTTP_METHODS
-from sanic.exceptions import InvalidUsage, NotFound, ServerError
+from sanic.exceptions import (
+    InvalidUsage,
+    NotFound,
+    SanicException,
+    ServerError,
+)
 from sanic.request import Request
 from sanic.response import json, text
 from sanic.views import CompositionView
@@ -18,6 +23,33 @@ from sanic.views import CompositionView
 # ------------------------------------------------------------ #
 
 
+def test_bp(app):
+    bp = Blueprint("test_text")
+
+    @bp.route("/")
+    def handler(request):
+        return text("Hello")
+
+    app.blueprint(bp)
+    request, response = app.test_client.get("/")
+
+    assert response.text == "Hello"
+
+
+def test_bp_app_access(app):
+    bp = Blueprint("test")
+
+    with pytest.raises(
+        SanicException,
+        match="<Blueprint test> has not yet been registered to an app",
+    ):
+        bp.apps
+
+    app.blueprint(bp)
+
+    assert app in bp.apps
+
+
 @pytest.fixture(scope="module")
 def static_file_directory():
     """The static directory to serve"""
@@ -62,20 +94,6 @@ def test_versioned_routes_get(app, method):
     assert response.status == 200
 
 
-def test_bp(app):
-    bp = Blueprint("test_text")
-
-    @bp.route("/")
-    def handler(request):
-        return text("Hello")
-
-    app.blueprint(bp)
-    request, response = app.test_client.get("/")
-    assert app.is_request_stream is False
-
-    assert response.text == "Hello"
-
-
 def test_bp_strict_slash(app):
     bp = Blueprint("test_text")
 
@@ -210,18 +228,28 @@ def test_bp_with_host(app):
 
     app.blueprint(bp)
     headers = {"Host": "example.com"}
+
     request, response = app.test_client.get("/test1/", headers=headers)
     assert response.text == "Hello"
 
     headers = {"Host": "sub.example.com"}
     request, response = app.test_client.get("/test1/", headers=headers)
-
-    assert response.text == "Hello subdomain!"
+    assert response.body == b"Hello subdomain!"
 
 
 def test_several_bp_with_host(app):
-    bp = Blueprint("test_text", url_prefix="/test", host="example.com")
-    bp2 = Blueprint("test_text2", url_prefix="/test", host="sub.example.com")
+    bp = Blueprint(
+        "test_text",
+        url_prefix="/test",
+        host="example.com",
+        strict_slashes=True,
+    )
+    bp2 = Blueprint(
+        "test_text2",
+        url_prefix="/test",
+        host="sub.example.com",
+        strict_slashes=True,
+    )
 
     @bp.route("/")
     def handler(request):
@@ -241,6 +269,7 @@ def test_several_bp_with_host(app):
     assert bp.host == "example.com"
     headers = {"Host": "example.com"}
     request, response = app.test_client.get("/test/", headers=headers)
+
     assert response.text == "Hello"
 
     assert bp2.host == "sub.example.com"
@@ -353,9 +382,32 @@ def test_bp_middleware(app):
     assert response.text == "FAIL"
 
 
+def test_bp_middleware_with_route(app):
+    blueprint = Blueprint("test_bp_middleware")
+
+    @blueprint.middleware("response")
+    async def process_response(request, response):
+        return text("OK")
+
+    @app.route("/")
+    async def handler(request):
+        return text("FAIL")
+
+    @blueprint.route("/bp")
+    async def bp_handler(request):
+        return text("FAIL")
+
+    app.blueprint(blueprint)
+
+    request, response = app.test_client.get("/bp")
+
+    assert response.status == 200
+    assert response.text == "OK"
+
+
 def test_bp_middleware_order(app):
     blueprint = Blueprint("test_bp_middleware_order")
-    order = list()
+    order = []
 
     @blueprint.middleware("request")
     def mw_1(request):
@@ -426,6 +478,7 @@ def test_bp_exception_handler(app):
 
 
 def test_bp_listeners(app):
+    app.route("/")(lambda x: x)
     blueprint = Blueprint("test_middleware")
 
     order = []
@@ -505,62 +558,52 @@ def test_bp_shorthand(app):
 
     @blueprint.get("/get")
     def handler(request):
-        assert request.stream is None
         return text("OK")
 
     @blueprint.put("/put")
     def put_handler(request):
-        assert request.stream is None
         return text("OK")
 
     @blueprint.post("/post")
     def post_handler(request):
-        assert request.stream is None
         return text("OK")
 
     @blueprint.head("/head")
     def head_handler(request):
-        assert request.stream is None
         return text("OK")
 
     @blueprint.options("/options")
     def options_handler(request):
-        assert request.stream is None
         return text("OK")
 
     @blueprint.patch("/patch")
     def patch_handler(request):
-        assert request.stream is None
         return text("OK")
 
     @blueprint.delete("/delete")
     def delete_handler(request):
-        assert request.stream is None
         return text("OK")
 
     @blueprint.websocket("/ws/", strict_slashes=True)
     async def websocket_handler(request, ws):
-        assert request.stream is None
         ev.set()
 
     app.blueprint(blueprint)
 
-    assert app.is_request_stream is False
-
     request, response = app.test_client.get("/get")
-    assert response.text == "OK"
+    assert response.body == b"OK"
 
     request, response = app.test_client.post("/get")
     assert response.status == 405
 
     request, response = app.test_client.put("/put")
-    assert response.text == "OK"
+    assert response.body == b"OK"
 
     request, response = app.test_client.get("/post")
     assert response.status == 405
 
     request, response = app.test_client.post("/post")
-    assert response.text == "OK"
+    assert response.body == b"OK"
 
     request, response = app.test_client.get("/post")
     assert response.status == 405
@@ -572,19 +615,19 @@ def test_bp_shorthand(app):
     assert response.status == 405
 
     request, response = app.test_client.options("/options")
-    assert response.text == "OK"
+    assert response.body == b"OK"
 
     request, response = app.test_client.get("/options")
     assert response.status == 405
 
     request, response = app.test_client.patch("/patch")
-    assert response.text == "OK"
+    assert response.body == b"OK"
 
     request, response = app.test_client.get("/patch")
     assert response.status == 405
 
     request, response = app.test_client.delete("/delete")
-    assert response.text == "OK"
+    assert response.body == b"OK"
 
     request, response = app.test_client.get("/delete")
     assert response.status == 405
@@ -710,7 +753,8 @@ def test_blueprint_middleware_with_args(app: Sanic):
 
 
 @pytest.mark.parametrize("file_name", ["test.file"])
-def test_static_blueprint_name(app: Sanic, static_file_directory, file_name):
+def test_static_blueprint_name(static_file_directory, file_name):
+    app = Sanic("app")
     current_file = inspect.getfile(inspect.currentframe())
     with open(current_file, "rb") as file:
         file.read()
@@ -727,7 +771,7 @@ def test_static_blueprint_name(app: Sanic, static_file_directory, file_name):
     app.blueprint(bp)
 
     uri = app.url_for("static", name="static.testing")
-    assert uri == "/static/test.file"
+    assert uri == "/static/test.file/"
 
     _, response = app.test_client.get("/static/test.file")
     assert response.status == 404
@@ -825,17 +869,19 @@ def test_duplicate_blueprint(app):
     )
 
 
-def test_strict_slashes_behavior_adoption(app):
+def test_strict_slashes_behavior_adoption():
+    app = Sanic("app")
     app.strict_slashes = True
+    bp = Blueprint("bp")
+    bp2 = Blueprint("bp2", strict_slashes=False)
 
     @app.get("/test")
     def handler_test(request):
         return text("Test")
 
-    assert app.test_client.get("/test")[1].status == 200
-    assert app.test_client.get("/test/")[1].status == 404
-
-    bp = Blueprint("bp")
+    @app.get("/f1", strict_slashes=False)
+    def f1(request):
+        return text("f1")
 
     @bp.get("/one", strict_slashes=False)
     def one(request):
@@ -845,7 +891,15 @@ def test_strict_slashes_behavior_adoption(app):
     def second(request):
         return text("second")
 
+    @bp2.get("/third")
+    def third(request):
+        return text("third")
+
     app.blueprint(bp)
+    app.blueprint(bp2)
+
+    assert app.test_client.get("/test")[1].status == 200
+    assert app.test_client.get("/test/")[1].status == 404
 
     assert app.test_client.get("/one")[1].status == 200
     assert app.test_client.get("/one/")[1].status == 200
@@ -853,19 +907,133 @@ def test_strict_slashes_behavior_adoption(app):
     assert app.test_client.get("/second")[1].status == 200
     assert app.test_client.get("/second/")[1].status == 404
 
-    bp2 = Blueprint("bp2", strict_slashes=False)
-
-    @bp2.get("/third")
-    def third(request):
-        return text("third")
-
-    app.blueprint(bp2)
     assert app.test_client.get("/third")[1].status == 200
     assert app.test_client.get("/third/")[1].status == 200
 
-    @app.get("/f1", strict_slashes=False)
-    def f1(request):
-        return text("f1")
-
     assert app.test_client.get("/f1")[1].status == 200
     assert app.test_client.get("/f1/")[1].status == 200
+
+
+def test_blueprint_group_versioning():
+    app = Sanic(name="blueprint-group-test")
+
+    bp1 = Blueprint(name="bp1", url_prefix="/bp1")
+    bp2 = Blueprint(name="bp2", url_prefix="/bp2", version=2)
+
+    bp3 = Blueprint(name="bp3", url_prefix="/bp3")
+
+    @bp3.get("/r1")
+    async def bp3_r1(request):
+        return json({"from": "bp3/r1"})
+
+    @bp1.get("/pre-group")
+    async def pre_group(request):
+        return json({"from": "bp1/pre-group"})
+
+    group = Blueprint.group([bp1, bp2], url_prefix="/group1", version=1)
+
+    group2 = Blueprint.group([bp3])
+
+    @bp1.get("/r1")
+    async def r1(request):
+        return json({"from": "bp1/r1"})
+
+    @bp2.get("/r2")
+    async def r2(request):
+        return json({"from": "bp2/r2"})
+
+    @bp2.get("/r3", version=3)
+    async def r3(request):
+        return json({"from": "bp2/r3"})
+
+    app.blueprint([group, group2])
+
+    assert app.test_client.get("/v1/group1/bp1/r1/")[1].status == 200
+    assert app.test_client.get("/v2/group1/bp2/r2")[1].status == 200
+    assert app.test_client.get("/v1/group1/bp1/pre-group")[1].status == 200
+    assert app.test_client.get("/v3/group1/bp2/r3")[1].status == 200
+    assert app.test_client.get("/bp3/r1")[1].status == 200
+
+    assert group.version == 1
+    assert group2.strict_slashes is None
+
+
+def test_blueprint_group_strict_slashes():
+    app = Sanic(name="blueprint-group-test")
+    bp1 = Blueprint(name="bp1", url_prefix=None, strict_slashes=False)
+
+    bp2 = Blueprint(
+        name="bp2", version=3, url_prefix="/bp2", strict_slashes=None
+    )
+
+    bp3 = Blueprint(
+        name="bp3", version=None, url_prefix="/bp3/", strict_slashes=None
+    )
+
+    @bp1.get("/r1")
+    async def bp1_r1(request):
+        return json({"from": "bp1/r1"})
+
+    @bp2.get("/r1")
+    async def bp2_r1(request):
+        return json({"from": "bp2/r1"})
+
+    @bp2.get("/r2/")
+    async def bp2_r2(request):
+        return json({"from": "bp2/r2"})
+
+    @bp3.get("/r1")
+    async def bp3_r1(request):
+        return json({"from": "bp3/r1"})
+
+    group = Blueprint.group(
+        [bp1, bp2],
+        url_prefix="/slash-check/",
+        version=1.3,
+        strict_slashes=True,
+    )
+
+    group2 = Blueprint.group(
+        [bp3], url_prefix="/other-prefix/", version="v2", strict_slashes=False
+    )
+
+    app.blueprint(group)
+    app.blueprint(group2)
+
+    assert app.test_client.get("/v1.3/slash-check/r1")[1].status == 200
+    assert app.test_client.get("/v1.3/slash-check/r1/")[1].status == 200
+    assert app.test_client.get("/v3/slash-check/bp2/r1")[1].status == 200
+    assert app.test_client.get("/v3/slash-check/bp2/r1/")[1].status == 404
+    assert app.test_client.get("/v3/slash-check/bp2/r2")[1].status == 404
+    assert app.test_client.get("/v3/slash-check/bp2/r2/")[1].status == 200
+    assert app.test_client.get("/v2/other-prefix/bp3/r1")[1].status == 200
+
+
+def test_blueprint_registered_multiple_apps():
+    app1 = Sanic("app1")
+    app2 = Sanic("app2")
+    bp = Blueprint("bp")
+
+    @bp.get("/")
+    async def handler(request):
+        return text(request.route.name)
+
+    app1.blueprint(bp)
+    app2.blueprint(bp)
+
+    for app in (app1, app2):
+        _, response = app.test_client.get("/")
+        assert response.text == f"{app.name}.bp.handler"
+
+
+def test_bp_set_attribute_warning():
+    bp = Blueprint("bp")
+    with pytest.warns(UserWarning) as record:
+        bp.foo = 1
+
+    assert len(record) == 1
+    assert record[0].message.args[0] == (
+        "Setting variables on Blueprint instances is deprecated "
+        "and will be removed in version 21.9. You should change your "
+        "Blueprint instance to use instance.ctx.foo instead."
+    )
