@@ -1,14 +1,26 @@
+import asyncio
+
 from bs4 import BeautifulSoup
 
 from sanic import Sanic
-from sanic.exceptions import InvalidUsage, NotFound, ServerError
+from sanic.exceptions import Forbidden, InvalidUsage, NotFound, ServerError
 from sanic.handlers import ErrorHandler
-from sanic.response import text
+from sanic.response import stream, text
 
 
 exception_handler_app = Sanic("test_exception_handler")
 
 
+async def sample_streaming_fn(response):
+    await response.write("foo,")
+    await asyncio.sleep(0.001)
+    await response.write("bar")
+
+
+class ErrorWithRequestCtx(ServerError):
+    pass
+
+
 @exception_handler_app.route("/1")
 def handler_1(request):
     raise InvalidUsage("OK")
@@ -47,11 +59,40 @@ def handler_6(request, arg):
     return text(foo)
 
 
-@exception_handler_app.exception(NotFound, ServerError)
+@exception_handler_app.route("/7")
+def handler_7(request):
+    raise Forbidden("go away!")
+
+
+@exception_handler_app.route("/8")
+def handler_8(request):
+
+    raise ErrorWithRequestCtx("OK")
+
+
+@exception_handler_app.exception(ErrorWithRequestCtx, NotFound)
+def handler_exception_with_ctx(request, exception):
+    return text(request.ctx.middleware_ran)
+
+
+@exception_handler_app.exception(ServerError)
 def handler_exception(request, exception):
     return text("OK")
 
 
+@exception_handler_app.exception(Forbidden)
+async def async_handler_exception(request, exception):
+    return stream(
+        sample_streaming_fn,
+        content_type="text/csv",
+    )
+
+
+@exception_handler_app.middleware
+async def some_request_middleware(request):
+    request.ctx.middleware_ran = "Done."
+
+
 def test_invalid_usage_exception_handler():
     request, response = exception_handler_app.test_client.get("/1")
     assert response.status == 400
@@ -71,7 +112,13 @@ def test_not_found_exception_handler():
 def test_text_exception__handler():
     request, response = exception_handler_app.test_client.get("/random")
     assert response.status == 200
-    assert response.text == "OK"
+    assert response.text == "Done."
+
+
+def test_async_exception_handler():
+    request, response = exception_handler_app.test_client.get("/7")
+    assert response.status == 200
+    assert response.text == "foo,bar"
 
 
 def test_html_traceback_output_in_debug_mode():
@@ -80,7 +127,7 @@ def test_html_traceback_output_in_debug_mode():
     soup = BeautifulSoup(response.body, "html.parser")
     html = str(soup)
 
-    assert "response = handler(request, *args, **kwargs)" in html
+    assert "response = handler(request, **kwargs)" in html
     assert "handler_4" in html
     assert "foo = bar" in html
 
@@ -104,7 +151,7 @@ def test_chained_exception_handler():
     soup = BeautifulSoup(response.body, "html.parser")
     html = str(soup)
 
-    assert "response = handler(request, *args, **kwargs)" in html
+    assert "response = handler(request, **kwargs)" in html
     assert "handler_6" in html
     assert "foo = 1 / arg" in html
     assert "ValueError" in html
@@ -156,3 +203,9 @@ def test_exception_handler_lookup():
     assert handler.lookup(CustomError()) == custom_error_handler
     assert handler.lookup(ServerError("Error")) == server_error_handler
     assert handler.lookup(CustomServerError("Error")) == server_error_handler
+
+
+def test_exception_handler_processed_request_middleware():
+    request, response = exception_handler_app.test_client.get("/8")
+    assert response.status == 200
+    assert response.text == "Done."
