@@ -1,17 +1,187 @@
 import asyncio
+import re
+
+from unittest.mock import Mock
 
 import pytest
 
-from sanic import Sanic
+from sanic_routing.exceptions import (
+    InvalidUsage,
+    ParameterNameConflicts,
+    RouteExists,
+)
+from sanic_testing.testing import SanicTestClient
+
+from sanic import Blueprint, Sanic
 from sanic.constants import HTTP_METHODS
+from sanic.exceptions import NotFound, SanicException
+from sanic.request import Request
 from sanic.response import json, text
-from sanic.router import ParameterNameConflicts, RouteDoesNotExist, RouteExists
-from sanic.testing import SanicTestClient
 
 
-# ------------------------------------------------------------ #
-#  UTF-8
-# ------------------------------------------------------------ #
+@pytest.mark.parametrize(
+    "path,headers,expected",
+    (
+        # app base
+        (b"/", {}, 200),
+        (b"/", {"host": "maybe.com"}, 200),
+        (b"/host", {"host": "matching.com"}, 200),
+        (b"/host", {"host": "wrong.com"}, 404),
+        # app strict_slashes default
+        (b"/without", {}, 200),
+        (b"/without/", {}, 200),
+        (b"/with", {}, 200),
+        (b"/with/", {}, 200),
+        # app strict_slashes off - expressly
+        (b"/expwithout", {}, 200),
+        (b"/expwithout/", {}, 200),
+        (b"/expwith", {}, 200),
+        (b"/expwith/", {}, 200),
+        # app strict_slashes on
+        (b"/without/strict", {}, 200),
+        (b"/without/strict/", {}, 404),
+        (b"/with/strict", {}, 404),
+        (b"/with/strict/", {}, 200),
+        # bp1 base
+        (b"/bp1", {}, 200),
+        (b"/bp1", {"host": "maybe.com"}, 200),
+        (b"/bp1/host", {"host": "matching.com"}, 200),  # BROKEN ON MASTER
+        (b"/bp1/host", {"host": "wrong.com"}, 404),
+        # bp1 strict_slashes default
+        (b"/bp1/without", {}, 200),
+        (b"/bp1/without/", {}, 200),
+        (b"/bp1/with", {}, 200),
+        (b"/bp1/with/", {}, 200),
+        # bp1 strict_slashes off - expressly
+        (b"/bp1/expwithout", {}, 200),
+        (b"/bp1/expwithout/", {}, 200),
+        (b"/bp1/expwith", {}, 200),
+        (b"/bp1/expwith/", {}, 200),
+        # bp1 strict_slashes on
+        (b"/bp1/without/strict", {}, 200),
+        (b"/bp1/without/strict/", {}, 404),
+        (b"/bp1/with/strict", {}, 404),
+        (b"/bp1/with/strict/", {}, 200),
+        # bp2 base
+        (b"/bp2/", {}, 200),
+        (b"/bp2/", {"host": "maybe.com"}, 200),
+        (b"/bp2/host", {"host": "matching.com"}, 200),  # BROKEN ON MASTER
+        (b"/bp2/host", {"host": "wrong.com"}, 404),
+        # bp2 strict_slashes default
+        (b"/bp2/without", {}, 200),
+        (b"/bp2/without/", {}, 404),
+        (b"/bp2/with", {}, 404),
+        (b"/bp2/with/", {}, 200),
+        # # bp2 strict_slashes off - expressly
+        (b"/bp2/expwithout", {}, 200),
+        (b"/bp2/expwithout/", {}, 200),
+        (b"/bp2/expwith", {}, 200),
+        (b"/bp2/expwith/", {}, 200),
+        # # bp2 strict_slashes on
+        (b"/bp2/without/strict", {}, 200),
+        (b"/bp2/without/strict/", {}, 404),
+        (b"/bp2/with/strict", {}, 404),
+        (b"/bp2/with/strict/", {}, 200),
+        # bp3 base
+        (b"/bp3", {}, 200),
+        (b"/bp3", {"host": "maybe.com"}, 200),
+        (b"/bp3/host", {"host": "matching.com"}, 200),  # BROKEN ON MASTER
+        (b"/bp3/host", {"host": "wrong.com"}, 404),
+        # bp3 strict_slashes default
+        (b"/bp3/without", {}, 200),
+        (b"/bp3/without/", {}, 200),
+        (b"/bp3/with", {}, 200),
+        (b"/bp3/with/", {}, 200),
+        # bp3 strict_slashes off - expressly
+        (b"/bp3/expwithout", {}, 200),
+        (b"/bp3/expwithout/", {}, 200),
+        (b"/bp3/expwith", {}, 200),
+        (b"/bp3/expwith/", {}, 200),
+        # bp3 strict_slashes on
+        (b"/bp3/without/strict", {}, 200),
+        (b"/bp3/without/strict/", {}, 404),
+        (b"/bp3/with/strict", {}, 404),
+        (b"/bp3/with/strict/", {}, 200),
+        # bp4 base
+        (b"/bp4", {}, 404),
+        (b"/bp4", {"host": "maybe.com"}, 200),
+        (b"/bp4/host", {"host": "matching.com"}, 200),  # BROKEN ON MASTER
+        (b"/bp4/host", {"host": "wrong.com"}, 404),
+        # bp4 strict_slashes default
+        (b"/bp4/without", {}, 404),
+        (b"/bp4/without/", {}, 404),
+        (b"/bp4/with", {}, 404),
+        (b"/bp4/with/", {}, 404),
+        # bp4 strict_slashes off - expressly
+        (b"/bp4/expwithout", {}, 404),
+        (b"/bp4/expwithout/", {}, 404),
+        (b"/bp4/expwith", {}, 404),
+        (b"/bp4/expwith/", {}, 404),
+        # bp4 strict_slashes on
+        (b"/bp4/without/strict", {}, 404),
+        (b"/bp4/without/strict/", {}, 404),
+        (b"/bp4/with/strict", {}, 404),
+        (b"/bp4/with/strict/", {}, 404),
+    ),
+)
+def test_matching(path, headers, expected):
+    app = Sanic("dev")
+    bp1 = Blueprint("bp1", url_prefix="/bp1")
+    bp2 = Blueprint("bp2", url_prefix="/bp2", strict_slashes=True)
+    bp3 = Blueprint("bp3", url_prefix="/bp3", strict_slashes=False)
+    bp4 = Blueprint("bp4", url_prefix="/bp4", host="maybe.com")
+
+    def handler(request):
+        return text("Hello!")
+
+    defs = (
+        ("/", None, None),
+        ("/host", None, "matching.com"),
+        ("/without", None, None),
+        ("/with/", None, None),
+        ("/expwithout", False, None),
+        ("/expwith/", False, None),
+        ("/without/strict", True, None),
+        ("/with/strict/", True, None),
+    )
+    for uri, strict_slashes, host in defs:
+        params = {"uri": uri}
+        if strict_slashes is not None:
+            params["strict_slashes"] = strict_slashes
+        if host is not None:
+            params["host"] = host
+        app.route(**params)(handler)
+        bp1.route(**params)(handler)
+        bp2.route(**params)(handler)
+        bp3.route(**params)(handler)
+        bp4.route(**params)(handler)
+
+    app.blueprint(bp1)
+    app.blueprint(bp2)
+    app.blueprint(bp3)
+    app.blueprint(bp4)
+
+    app.router.finalize()
+
+    request = Request(path, headers, None, "GET", None, app)
+
+    try:
+        app.router.get(
+            request.path, request.method, request.headers.get("host")
+        )
+    except NotFound:
+        response = 404
+    except Exception:
+        response = 500
+    else:
+        response = 200
+
+    assert response == expected
+
+
+# # ------------------------------------------------------------ #
+# #  UTF-8
+# # ------------------------------------------------------------ #
 
 
 @pytest.mark.parametrize("method", HTTP_METHODS)
@@ -26,7 +196,6 @@ def test_versioned_routes_get(app, method):
             return text("OK")
 
     else:
-        print(func)
         raise Exception(f"Method: {method} is not callable")
 
     client_method = getattr(app.test_client, method)
@@ -67,16 +236,12 @@ def test_shorthand_routes_multiple(app):
 def test_route_strict_slash(app):
     @app.get("/get", strict_slashes=True)
     def handler1(request):
-        assert request.stream is None
         return text("OK")
 
     @app.post("/post/", strict_slashes=True)
     def handler2(request):
-        assert request.stream is None
         return text("OK")
 
-    assert app.is_request_stream is False
-
     request, response = app.test_client.get("/get")
     assert response.text == "OK"
 
@@ -167,7 +332,6 @@ def test_route_optional_slash(app):
 
 def test_route_strict_slashes_set_to_false_and_host_is_a_list(app):
     # Part of regression test for issue #1120
-
     test_client = SanicTestClient(app, port=42101)
     site1 = f"127.0.0.1:{test_client.port}"
 
@@ -179,6 +343,8 @@ def test_route_strict_slashes_set_to_false_and_host_is_a_list(app):
     request, response = test_client.get("http://" + site1 + "/get")
     assert response.text == "OK"
 
+    app.router.finalized = False
+
     @app.post("/post", host=[site1, "site2.com"], strict_slashes=False)
     def post_handler(request):
         return text("OK")
@@ -186,6 +352,8 @@ def test_route_strict_slashes_set_to_false_and_host_is_a_list(app):
     request, response = test_client.post("http://" + site1 + "/post")
     assert response.text == "OK"
 
+    app.router.finalized = False
+
     @app.put("/put", host=[site1, "site2.com"], strict_slashes=False)
     def put_handler(request):
         return text("OK")
@@ -193,6 +361,8 @@ def test_route_strict_slashes_set_to_false_and_host_is_a_list(app):
     request, response = test_client.put("http://" + site1 + "/put")
     assert response.text == "OK"
 
+    app.router.finalized = False
+
     @app.delete("/delete", host=[site1, "site2.com"], strict_slashes=False)
     def delete_handler(request):
         return text("OK")
@@ -216,11 +386,8 @@ def test_shorthand_routes_post(app):
 def test_shorthand_routes_put(app):
     @app.put("/put")
     def handler(request):
-        assert request.stream is None
         return text("OK")
 
-    assert app.is_request_stream is False
-
     request, response = app.test_client.put("/put")
     assert response.text == "OK"
 
@@ -231,11 +398,8 @@ def test_shorthand_routes_put(app):
 def test_shorthand_routes_delete(app):
     @app.delete("/delete")
     def handler(request):
-        assert request.stream is None
         return text("OK")
 
-    assert app.is_request_stream is False
-
     request, response = app.test_client.delete("/delete")
     assert response.text == "OK"
 
@@ -246,11 +410,8 @@ def test_shorthand_routes_delete(app):
 def test_shorthand_routes_patch(app):
     @app.patch("/patch")
     def handler(request):
-        assert request.stream is None
         return text("OK")
 
-    assert app.is_request_stream is False
-
     request, response = app.test_client.patch("/patch")
     assert response.text == "OK"
 
@@ -261,11 +422,8 @@ def test_shorthand_routes_patch(app):
 def test_shorthand_routes_head(app):
     @app.head("/head")
     def handler(request):
-        assert request.stream is None
         return text("OK")
 
-    assert app.is_request_stream is False
-
     request, response = app.test_client.head("/head")
     assert response.status == 200
 
@@ -276,11 +434,8 @@ def test_shorthand_routes_head(app):
 def test_shorthand_routes_options(app):
     @app.options("/options")
     def handler(request):
-        assert request.stream is None
         return text("OK")
 
-    assert app.is_request_stream is False
-
     request, response = app.test_client.options("/options")
     assert response.status == 200
 
@@ -312,6 +467,8 @@ def test_dynamic_route(app):
         results.append(name)
         return text("OK")
 
+    app.router.finalize(False)
+
     request, response = app.test_client.get("/folder/test123")
 
     assert response.text == "OK"
@@ -386,6 +543,9 @@ def test_dynamic_route_regex(app):
     async def handler(request, folder_id):
         return text("OK")
 
+    app.router.finalize()
+    print(app.router.find_route_src)
+
     request, response = app.test_client.get("/folder/test")
     assert response.status == 200
 
@@ -433,6 +593,8 @@ def test_dynamic_route_path(app):
     request, response = app.test_client.get("/info")
     assert response.status == 404
 
+    app.router.reset()
+
     @app.route("/<path:path>")
     async def handler1(request, path):
         return text("OK")
@@ -477,6 +639,19 @@ def test_websocket_route(app, url):
     assert ev.is_set()
 
 
+def test_websocket_route_invalid_handler(app):
+    with pytest.raises(ValueError) as e:
+
+        @app.websocket("/")
+        async def handler():
+            ...
+
+    assert e.match(
+        r"Required parameter `request` and/or `ws` missing in the "
+        r"handler\(\) route\?"
+    )
+
+
 @pytest.mark.asyncio
 @pytest.mark.parametrize("url", ["/ws", "ws"])
 async def test_websocket_route_asgi(app, url):
@@ -498,21 +673,21 @@ def test_websocket_route_with_subprotocols(app):
         results.append(ws.subprotocol)
         assert ws.subprotocol is not None
 
-    request, response = app.test_client.websocket("/ws", subprotocols=["bar"])
+    _, response = SanicTestClient(app).websocket("/ws", subprotocols=["bar"])
     assert response.opened is True
     assert results == ["bar"]
 
-    request, response = app.test_client.websocket(
+    _, response = SanicTestClient(app).websocket(
         "/ws", subprotocols=["bar", "foo"]
     )
     assert response.opened is True
     assert results == ["bar", "bar"]
 
-    request, response = app.test_client.websocket("/ws", subprotocols=["baz"])
+    _, response = SanicTestClient(app).websocket("/ws", subprotocols=["baz"])
     assert response.opened is True
     assert results == ["bar", "bar", None]
 
-    request, response = app.test_client.websocket("/ws")
+    _, response = SanicTestClient(app).websocket("/ws")
     assert response.opened is True
     assert results == ["bar", "bar", None, None]
 
@@ -792,7 +967,7 @@ def test_removing_slash(app):
     def post(_):
         pass
 
-    assert len(app.router.routes_all.keys()) == 2
+    assert len(app.router.routes_all.keys()) == 1
 
 
 def test_overload_routes(app):
@@ -816,6 +991,7 @@ def test_overload_routes(app):
     request, response = app.test_client.delete("/overload")
     assert response.status == 405
 
+    app.router.reset()
     with pytest.raises(RouteExists):
 
         @app.route("/overload", methods=["PUT", "DELETE"])
@@ -828,11 +1004,18 @@ def test_unmergeable_overload_routes(app):
     async def handler1(request):
         return text("OK1")
 
-    with pytest.raises(RouteExists):
+    @app.route("/overload_whole", methods=["POST", "PUT"])
+    async def handler2(request):
+        return text("OK1")
 
-        @app.route("/overload_whole", methods=["POST", "PUT"])
-        async def handler2(request):
-            return text("Duplicated")
+    assert (
+        len(
+            dict(list(app.router.static_routes.values())[0].handlers)[
+                "overload_whole"
+            ]
+        )
+        == 3
+    )
 
     request, response = app.test_client.get("/overload_whole")
     assert response.text == "OK1"
@@ -840,6 +1023,11 @@ def test_unmergeable_overload_routes(app):
     request, response = app.test_client.post("/overload_whole")
     assert response.text == "OK1"
 
+    request, response = app.test_client.put("/overload_whole")
+    assert response.text == "OK1"
+
+    app.router.reset()
+
     @app.route("/overload_part", methods=["GET"])
     async def handler3(request):
         return text("OK1")
@@ -865,7 +1053,9 @@ def test_unicode_routes(app):
     request, response = app.test_client.get("/你好")
     assert response.text == "OK1"
 
-    @app.route("/overload/<param>", methods=["GET"])
+    app.router.reset()
+
+    @app.route("/overload/<param>", methods=["GET"], unquote=True)
     async def handler2(request, param):
         return text("OK2 " + param)
 
@@ -883,20 +1073,38 @@ def test_uri_with_different_method_and_different_params(app):
         return json({"action": action})
 
     request, response = app.test_client.get("/ads/1234")
-    assert response.status == 200
-    assert response.json == {"ad_id": "1234"}
+    assert response.status == 405
 
     request, response = app.test_client.post("/ads/post")
     assert response.status == 200
     assert response.json == {"action": "post"}
 
 
-def test_route_raise_ParameterNameConflicts(app):
-    with pytest.raises(ParameterNameConflicts):
+def test_uri_with_different_method_and_same_params(app):
+    @app.route("/ads/<ad_id>", methods=["GET"])
+    async def ad_get(request, ad_id):
+        return json({"ad_id": ad_id})
 
-        @app.get("/api/v1/<user>/<user>/")
-        def handler(request, user):
-            return text("OK")
+    @app.route("/ads/<ad_id>", methods=["POST"])
+    async def ad_post(request, ad_id):
+        return json({"ad_id": ad_id})
+
+    request, response = app.test_client.get("/ads/1234")
+    assert response.status == 200
+    assert response.json == {"ad_id": "1234"}
+
+    request, response = app.test_client.post("/ads/post")
+    assert response.status == 200
+    assert response.json == {"ad_id": "post"}
+
+
+def test_route_raise_ParameterNameConflicts(app):
+    @app.get("/api/v1/<user>/<user>/")
+    def handler(request, user):
+        return text("OK")
+
+    with pytest.raises(ParameterNameConflicts):
+        app.router.finalize()
 
 
 def test_route_invalid_host(app):
@@ -911,3 +1119,115 @@ def test_route_invalid_host(app):
     assert str(excinfo.value) == (
         "Expected either string or Iterable of " "host strings, not {!r}"
     ).format(host)
+
+
+def test_route_with_regex_group(app):
+    @app.route("/path/to/<ext:file\.(txt)>")
+    async def handler(request, ext):
+        return text(ext)
+
+    _, response = app.test_client.get("/path/to/file.txt")
+    assert response.text == "txt"
+
+
+def test_route_with_regex_named_group(app):
+    @app.route(r"/path/to/<ext:file\.(?P<ext>txt)>")
+    async def handler(request, ext):
+        return text(ext)
+
+    _, response = app.test_client.get("/path/to/file.txt")
+    assert response.text == "txt"
+
+
+def test_route_with_regex_named_group_invalid(app):
+    @app.route(r"/path/to/<ext:file\.(?P<wrong>txt)>")
+    async def handler(request, ext):
+        return text(ext)
+
+    with pytest.raises(InvalidUsage) as e:
+        app.router.finalize()
+
+    assert e.match(
+        re.escape("Named group (wrong) must match your named parameter (ext)")
+    )
+
+
+def test_route_with_regex_group_ambiguous(app):
+    @app.route("/path/to/<ext:file(?:\.)(txt)>")
+    async def handler(request, ext):
+        return text(ext)
+
+    with pytest.raises(InvalidUsage) as e:
+        app.router.finalize()
+
+    assert e.match(
+        re.escape(
+            "Could not compile pattern file(?:\.)(txt). Try using a named "
+            "group instead: '(?P<ext>your_matching_group)'"
+        )
+    )
+
+
+def test_route_with_bad_named_param(app):
+    @app.route("/foo/<__bar__>")
+    async def handler(request):
+        return text("...")
+
+    with pytest.raises(SanicException):
+        app.router.finalize()
+
+
+def test_routes_with_and_without_slash_definitions(app):
+    bar = Blueprint("bar", url_prefix="bar")
+    baz = Blueprint("baz", url_prefix="/baz")
+    fizz = Blueprint("fizz", url_prefix="fizz/")
+    buzz = Blueprint("buzz", url_prefix="/buzz/")
+
+    instances = (
+        (app, "foo"),
+        (bar, "bar"),
+        (baz, "baz"),
+        (fizz, "fizz"),
+        (buzz, "buzz"),
+    )
+
+    for instance, term in instances:
+        route = f"/{term}" if isinstance(instance, Sanic) else ""
+
+        @instance.get(route, strict_slashes=True)
+        def get_without(request):
+            return text(f"{term}_without")
+
+        @instance.get(f"{route}/", strict_slashes=True)
+        def get_with(request):
+            return text(f"{term}_with")
+
+        @instance.post(route, strict_slashes=True)
+        def post_without(request):
+            return text(f"{term}_without")
+
+        @instance.post(f"{route}/", strict_slashes=True)
+        def post_with(request):
+            return text(f"{term}_with")
+
+    app.blueprint(bar)
+    app.blueprint(baz)
+    app.blueprint(fizz)
+    app.blueprint(buzz)
+
+    for _, term in instances:
+        _, response = app.test_client.get(f"/{term}")
+        assert response.status == 200
+        assert response.text == f"{term}_without"
+
+        _, response = app.test_client.get(f"/{term}/")
+        assert response.status == 200
+        assert response.text == f"{term}_with"
+
+        _, response = app.test_client.post(f"/{term}")
+        assert response.status == 200
+        assert response.text == f"{term}_without"
+
+        _, response = app.test_client.post(f"/{term}/")
+        assert response.status == 200
+        assert response.text == f"{term}_with"
