@@ -1,6 +1,7 @@
 import inspect
 import os
 
+from pathlib import Path
 from time import gmtime, strftime
 
 import pytest
@@ -76,6 +77,42 @@ def test_static_file(app, static_file_directory, file_name):
     assert response.body == get_file_content(static_file_directory, file_name)
 
 
+@pytest.mark.parametrize(
+    "file_name",
+    ["test.file", "decode me.txt", "python.png", "symlink", "hard_link"],
+)
+def test_static_file_pathlib(app, static_file_directory, file_name):
+    file_path = Path(get_file_path(static_file_directory, file_name))
+    app.static("/testing.file", file_path)
+    request, response = app.test_client.get("/testing.file")
+    assert response.status == 200
+    assert response.body == get_file_content(static_file_directory, file_name)
+
+
+@pytest.mark.parametrize(
+    "file_name",
+    [b"test.file", b"decode me.txt", b"python.png"],
+)
+def test_static_file_bytes(app, static_file_directory, file_name):
+    bsep = os.path.sep.encode("utf-8")
+    file_path = static_file_directory.encode("utf-8") + bsep + file_name
+    app.static("/testing.file", file_path)
+    request, response = app.test_client.get("/testing.file")
+    assert response.status == 200
+
+
+@pytest.mark.parametrize(
+    "file_name",
+    [{}, [], object()],
+)
+def test_static_file_invalid_path(app, static_file_directory, file_name):
+    app.route("/")(lambda x: x)
+    with pytest.raises(ValueError):
+        app.static("/testing.file", file_name)
+    request, response = app.test_client.get("/testing.file")
+    assert response.status == 404
+
+
 @pytest.mark.parametrize("file_name", ["test.html"])
 def test_static_file_content_type(app, static_file_directory, file_name):
     app.static(
@@ -90,6 +127,40 @@ def test_static_file_content_type(app, static_file_directory, file_name):
     assert response.headers["Content-Type"] == "text/html; charset=utf-8"
 
 
+@pytest.mark.parametrize(
+    "file_name,expected",
+    [
+        ("test.html", "text/html; charset=utf-8"),
+        ("decode me.txt", "text/plain; charset=utf-8"),
+        ("test.file", "application/octet-stream"),
+    ],
+)
+def test_static_file_content_type_guessed(
+    app, static_file_directory, file_name, expected
+):
+    app.static(
+        "/testing.file",
+        get_file_path(static_file_directory, file_name),
+    )
+
+    request, response = app.test_client.get("/testing.file")
+    assert response.status == 200
+    assert response.body == get_file_content(static_file_directory, file_name)
+    assert response.headers["Content-Type"] == expected
+
+
+def test_static_file_content_type_with_charset(app, static_file_directory):
+    app.static(
+        "/testing.file",
+        get_file_path(static_file_directory, "decode me.txt"),
+        content_type="text/plain;charset=ISO-8859-1",
+    )
+
+    request, response = app.test_client.get("/testing.file")
+    assert response.status == 200
+    assert response.headers["Content-Type"] == "text/plain;charset=ISO-8859-1"
+
+
 @pytest.mark.parametrize(
     "file_name", ["test.file", "decode me.txt", "symlink", "hard_link"]
 )
@@ -374,3 +445,12 @@ def test_static_name(app, static_file_directory, static_name, file_name):
     request, response = app.test_client.get(f"/static/{file_name}")
 
     assert response.status == 200
+
+
+def test_nested_dir(app, static_file_directory):
+    app.static("/static", static_file_directory)
+
+    request, response = app.test_client.get("/static/nested/dir/foo.txt")
+
+    assert response.status == 200
+    assert response.text == "foo\n"
